const USB = require('usb');
const Common = require('../../constants/common');
const I2C = require('../../constants/I2C');
const PID = Common.PID;
const VID = Common.VID;
const DEVICE = USB.findByIds(VID, PID);

const DATA = {
    bmRequestType: 0x40,
    bRequest: 0,
    wValue: 0,
    wIndex: 0,
    data_or_length: Buffer(0)
};

function init() {
    DEVICE.open();
}

function enable() {
    DATA.bRequest = I2C.I2C_BITBANG_ON;
    DEVICE.controlTransfer(DATA.bmRequestType, DATA.bRequest, DATA.wValue, DATA.wIndex, DATA.data_or_length, function (error) {
        if (error !== undefined) console.log("[BitBang enable]: " + error);
    });
}

function sdaHigh() {
    DATA.bRequest = I2C.I2C_SDA_HI;
    DEVICE.controlTransfer(DATA.bmRequestType, DATA.bRequest, DATA.wValue, DATA.wIndex, DATA.data_or_length, function (error) {
        if (error !== undefined) console.log("[SDA high]: " + error);
    });
}

function sdaLow() {
    DATA.bRequest = I2C.I2C_SDA_LO;
    DEVICE.controlTransfer(DATA.bmRequestType, DATA.bRequest, DATA.wValue, DATA.wIndex, DATA.data_or_length, function (error) {
        if (error !== undefined) console.log("[SDA low]: " + error);
    });
}

function sclHigh() {
    DATA.bRequest = I2C.I2C_SCL_HI;
    DEVICE.controlTransfer(DATA.bmRequestType, DATA.bRequest, DATA.wValue, DATA.wIndex, DATA.data_or_length, function (error) {
        if (error !== undefined) console.log("[SCL high]: " + error);
    });
}

function sclLow() {
    DATA.bRequest = I2C.I2C_SCL_LO;
    DEVICE.controlTransfer(DATA.bmRequestType, DATA.bRequest, DATA.wValue, DATA.wIndex, DATA.data_or_length, function (error) {
        if (error !== undefined) console.log("[SCL low]: " + error);
    });
}

function disable() {
    DATA.bRequest = I2C.I2C_BITBANG_OFF;
    DEVICE.controlTransfer(DATA.bmRequestType, DATA.bRequest, DATA.wValue, DATA.wIndex, DATA.data_or_length, function (error) {
        if (error !== undefined) console.log("[BitBang disable]: " + error);
    });
}

function close() {
    DEVICE.close();
}

module.exports = {
    init: init,
    enable: enable,
    sdaHigh: sdaHigh,
    sdaLow: sdaLow,
    sclHigh: sclHigh,
    sclLow: sclLow,
    disable: disable,
    close: close
};