const OLED = require('./lib/I2C/SSD1306/SSD1306');
const SSD1306 = require('./lib/constants/SSD1306');
const EXPRESS = require('express');
const APP = EXPRESS();
const HTTP = require('http').Server(APP);
const IO = require('socket.io')(HTTP);
const PORT = 3000;
const BLACK_COLOR = "#000000";
const BITRATE = 500;
let buffer = [];

OLED.init(SSD1306.SWITCHCAPVCC, SSD1306.REFRESH_MID, BITRATE);

APP.use('/', EXPRESS.static('./static'));

APP.get('/', function (req, res) {
    res.sendFile(__dirname + '/static/index.html');
});

IO.on('connection', function (socket) {
    console.log("Connected: " + socket.id);

    buffer = uniqueBy(buffer, (o1, o2) => o1.x === o2.x && o1.y === o2.y);
    socket.emit('echo', buffer);

    socket.on('update', function (msg) {
        const json = JSON.parse(msg);
        for (let i = 0; i < json.length; i++) {
            let color = (json[i].color === BLACK_COLOR) ? 0 : 1;
            OLED.setPixel(json[i].x, json[i].y, color);
            buffer.unshift(json[i]);
        }
        OLED.update();
        socket.broadcast.emit('echo', json);
    });

    socket.on('clear', function () {
        OLED.clear();
        OLED.update();
        buffer = [];
        socket.broadcast.emit('clear-canvas');
    });

});

HTTP.listen(PORT, function () {
    console.log('Listening on *:' + PORT);
});

function uniqueBy(a, cond) {
    return a.filter((e, i) => a.findIndex(e2 => cond(e, e2)) === i);
}
