$('document').ready(function () {
    const SCALE = 2; // it must be even
    const PIXEL_CENTER = SCALE / 2;
    const BLUE_COLOR = "#449af5";
    const BLACK_COLOR = "#000000";
    const YELLOW_COLOR = "#f0d615";
    const OLED_WIDTH = 128;
    const OLED_HEIGHT = 64;
    const UPDATE_TIME_MS = 250;

    const socket = io();

    const canvas = document.getElementById('canvas');
    const ctx = canvas.getContext('2d');
    const CANVAS_WIDTH = canvas.width;
    const CANVAS_HEIGHT = canvas.height;

    const textCanvas = document.createElement('canvas');
    textCanvas.setAttribute("width", CANVAS_WIDTH);
    textCanvas.setAttribute("height", CANVAS_HEIGHT);
    const textCtx = textCanvas.getContext('2d');
    textCtx.fillStyle = BLUE_COLOR;

    const drawCanvas = document.createElement('canvas');
    drawCanvas.setAttribute("width", CANVAS_WIDTH);
    drawCanvas.setAttribute("height", CANVAS_HEIGHT);
    const drawCtx = drawCanvas.getContext('2d');
    drawCtx.fillStyle = BLUE_COLOR;

    const tempCanvas = document.createElement('canvas');
    tempCanvas.setAttribute("width", CANVAS_WIDTH);
    tempCanvas.setAttribute("height", CANVAS_HEIGHT);
    const tempCtx = tempCanvas.getContext('2d');

    let isChanged = false;

    ctx.fillStyle = BLUE_COLOR;
    textCtx.fillStyle = BLUE_COLOR;
    drawCtx.fillStyle = BLUE_COLOR;

    $('#btn-clear').bind('click', () => {
        clear();
        socket.emit('clear', true);
    });

    $('#canvas').bind('touchmove', drawPoint);

    $('#canvas').on('scroll touchmove mousewheel', (e) => {
        e.preventDefault();
        e.stopPropagation();
    });

    $('#canvas').bind('mousedown', (e) => {
        drawPoint(e);
        $('#canvas').bind('mousemove', drawPoint);
        $('#canvas').bind('mouseup', function () {
            $('#canvas').unbind('mousemove', drawPoint);
        });
    });

    $('#btn-start').bind('click', function () {
        setInterval(updateDisplay, UPDATE_TIME_MS);
        $(this).prop('disabled', true);
    });

    $("#text-to-send").bind('keyup', drawText);


    $('#x-position, #y-position, #font-size, #font-type').bind('keyup change click mousewheel', () => {
        drawText();
    });

    socket.on('echo', (msg) => {
        msg.forEach((item) => {
            let x = item.x * SCALE;
            let y = item.y * SCALE;
            drawCtx.fillStyle = item.color;
            drawCtx.fillRect(x - PIXEL_CENTER, y - PIXEL_CENTER, SCALE, SCALE);
        });

        drawCtx.stroke();

        updateCtx();

        tempCtx.clearRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);
        tempCtx.drawImage(canvas, 0, 0);
    });

    socket.on('clear-canvas', () => {
        clear();
    });

    function clear() {
        ctx.clearRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);
        drawCtx.clearRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);
        textCtx.clearRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);
        tempCtx.clearRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);
        $('#text-to-send').val('');
    }

    function drawPoint(e) {
        const target = e.originalEvent;
        const brushSize = $('#brush-size').val() * SCALE;
        let x;
        let y;

        if (target.layerX !== undefined) {
            x = target.layerX;
            y = target.layerY;
        } else {
            let offset = $('#canvas').offset();
            x = target.targetTouches[0].pageX - offset.left;
            y = target.targetTouches[0].pageY - offset.top;
        }
        x = Math.floor(x / SCALE) * SCALE;
        y = Math.floor(y / SCALE) * SCALE;


        if (($('#erase-color').is(':checked'))) {
            drawCtx.fillStyle = BLACK_COLOR;
            drawCtx.fillRect(x - brushSize / 2, y - brushSize / 2, brushSize, brushSize);
            drawCtx.stroke();

            textCtx.fillStyle = BLACK_COLOR;
            textCtx.fillRect(x - brushSize / 2, y - brushSize / 2, brushSize, brushSize);
            textCtx.stroke();
        } else {
            drawCtx.fillStyle = BLUE_COLOR;
            drawCtx.fillRect(x - brushSize / 2, y - brushSize / 2, brushSize, brushSize);
            drawCtx.stroke();
        }

        updateCtx();
        isChanged = true;
    }

    function updateDisplay() {
        if (isChanged) {
            let pixels = [];

            for (let y = 0; y < OLED_HEIGHT; y++) {
                for (let x = 0; x < OLED_WIDTH; x++) {
                    if (ctx.getImageData(x * SCALE, y * SCALE, 1, 1).data[0] !== tempCtx.getImageData(x * SCALE, y * SCALE, 1, 1).data[0]) {
                        pixels.push({
                            x: x,
                            y: y,
                            color: rgbToHex(ctx.getImageData(x * SCALE, y * SCALE, 1, 1).data)
                        });
                    }
                }
            }

            if (pixels.length === 0) return;

            socket.emit('update', JSON.stringify(pixels));
            console.log(JSON.stringify(pixels));

            tempCtx.clearRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);
            tempCtx.drawImage(canvas, 0, 0);

            isChanged = false;
        }
    }

    function rgbToHex(data) {
        return "#" + ((1 << 24) + (data[0] << 16) + (data[1] << 8) + data[2]).toString(16).slice(1);
    }

    function drawText() {
        let x = $('#x-position').val() * SCALE;
        let y = $('#y-position').val() * SCALE;
        let size = $('#font-size').val() * SCALE;
        let font = $('#font-type').val();
        let textToSend = $('#text-to-send').val();

        textCtx.clearRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);
        textCtx.font = `${size}px ${font}`;
        textCtx.fillStyle = BLUE_COLOR;
        textCtx.fillText(textToSend, x, y);

        updateCtx();
        isChanged = true;
    }

    function updateCtx() {
        ctx.clearRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);

        for (let y = 0; y < CANVAS_HEIGHT; y += SCALE) {
            for (let x = 0; x < CANVAS_WIDTH; x += SCALE) {
                if (drawCtx.getImageData(x, y, 1, 1).data[0] || textCtx.getImageData(x, y, 1, 1).data[0]) {
                    ctx.fillStyle = (y > SCALE * 16) ? BLUE_COLOR : YELLOW_COLOR;
                    ctx.fillRect(x - 2, y - 2, SCALE, SCALE);
                }
            }
        }

        ctx.stroke();
    }

});


